/*
DISCLAIMER FOR PERSONAL IDENTITY VERIFICATION EXAMPLE CODE
May 2006

This software was developed by the National Institute of Standards and
Technology (NIST), an agency of the Federal Government. Pursuant to
Title 15 United States Code Section 105, works of NIST are not subject
to copyright protection in the United States and are considered to be
in the public domain. Thus, the software may be freely reproduced. 

This software is released by NIST as a service and is expressly provided "AS 
IS." NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY. NIST 
DOES NOT REPRESENT OR WARRANT THAT THE OPERATION OF THE SOFTWARE WILL BE 
UNINTERRUPTED OR ERROR-FREE, OR THAT ANY DEFECTS WILL BE CORRECTED. 

NIST does not warrant or make any representations regarding the use of the 
software or the results thereof, including but not limited to the 
correctness, accuracy, reliability or usefulness of the software. By using 
this software or by incorporating this software into another product, you 
agree to hold harmless the United Sates Government for any and all damages or 
liabilities that arise out of such use.
*/

/* file cspinstl.c
 *
 * NIST PIV Demo CSP -- Cryptographic Service Provider
 *
 * Designed and Developed by Electrosoft (www.electrosoft-inc.com) based on requirements
 * from the National Institute of Standards and Technology (NIST).
*/

#undef UNICODE					// ## Not Yet
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <windows.h>
#include <wincrypt.h>

CHAR szprovider[] = "SOFTWARE\\Microsoft\\Cryptography\\Defaults\\Provider\\NIST-ESI CSP";

#ifdef ISO24727
CHAR szSigPath[]   = "escsp_iso24727.sig";
CHAR szImagePath[] = "escsp_iso24727.dll";
#else
CHAR szSigPath[]   = "escsp.sig";
CHAR szImagePath[] = "escsp.dll";
#endif

DWORD     	    dwIgn;
HKEY      	    hKey;
DWORD           err;
DWORD           dwValue;
HANDLE          hFileSig;
DWORD     	    NumBytesRead;
DWORD           lpdwFileSizeHigh;
LPVOID          lpvAddress;
DWORD           NumBytes;

int __cdecl main(int cArg, char *rgszArg[])
{
    //
    // Just to open escsp.dll signature file.  This file was created by
    // sign.exe.
    //
    if ((hFileSig = CreateFile(
						szSigPath,
						GENERIC_READ, 
						0, 
						NULL,
						OPEN_EXISTING, 
						FILE_ATTRIBUTE_NORMAL,
						0)) != INVALID_HANDLE_VALUE)
    {
        if ((NumBytes = GetFileSize((HANDLE) hFileSig, &lpdwFileSizeHigh)) ==
                                    0xffffffff)
        {
            printf("Install failed: Getting size of file cspsign: %x\n",
                    GetLastError());
            CloseHandle(hFileSig);
            return(FALSE);
        }

        if ((lpvAddress = VirtualAlloc(
								NULL, 
								NumBytes, 
								MEM_RESERVE | MEM_COMMIT,
								PAGE_READWRITE)) == NULL)
        {
            CloseHandle(hFileSig);
            printf("Install failed: Alloc to read uisign: %x\n",
                    GetLastError());
            return(FALSE);
        }

        if (!ReadFile((HANDLE) hFileSig, lpvAddress, NumBytes,
		      &NumBytesRead, 0))
        {

            CloseHandle(hFileSig);
            printf("Install failed: Reading uisign: %x\n",
                    GetLastError());
            VirtualFree(lpvAddress, 0, MEM_RELEASE);
            return(FALSE);
        }

        CloseHandle(hFileSig);

        if (NumBytesRead != NumBytes)
        {
            printf("Install failed: Bytes read doesn't match file size\n");
            return(FALSE);
        }

		//
		// Create or open in local machine for provider:
		//
        if ((err = RegCreateKeyEx(
						HKEY_LOCAL_MACHINE,
						(const char *) szprovider,
						0L,
						"", 
						REG_OPTION_NON_VOLATILE,
						KEY_ALL_ACCESS, 
						NULL, 
						&hKey,
						&dwIgn)) != ERROR_SUCCESS)
        {
            printf("Install failed: RegCreateKeyEx\n");
        }

		//
		// Set Image path to: escsp.dll
		//
        if ((err = RegSetValueEx(
						hKey, 
						"Image Path", 
						0L, 
						REG_SZ, 
						szImagePath,
						strlen(szImagePath)+1)) != ERROR_SUCCESS)
        {
            printf("Install failed: Setting Image Path value\n");
            return(FALSE);
        }

		//
		// Set Type to: Type 1
		//
        dwValue = 1;
        if ((err = RegSetValueEx(
						hKey,
						"Type", 
						0L, 
						REG_DWORD,
						(LPTSTR) &dwValue,
						sizeof(DWORD))) != ERROR_SUCCESS)
        {
            printf("Install failed: Setting Type value: %x\n", err);
            return(FALSE);
        }

		//
		// Place signature
		//
        if ((err = RegSetValueEx(
						hKey,
						"Signature", 
						0L, 
						REG_BINARY,
						(LPTSTR) lpvAddress,
						NumBytes)) != ERROR_SUCCESS)
        {
            printf("Install failed: Setting Signature value for cspsign: %x\n", err);
            return(FALSE);
        }

        RegCloseKey(hKey);
        VirtualFree(lpvAddress, 0, MEM_RELEASE);

		printf("Installed: %s\n", szImagePath);
    }

    return(FALSE);
}
